Ext.define('Capybara.controller.DownloadManager', {
    extend: 'Ext.app.Controller',

    requires: [
        'Capybara.view.bulkexport.DownloadManagerGrid',
        'Capybara.view.navigation.NavigationMenu',
        'Capybara.ux.Notification',
        'Capybara.util.Utility',
        'Capybara.app.Constants',
        'Capybara.view.dialogs.BulkExportOnDemandDetailDialog',
        'Capybara.view.dialogs.BulkExportOnDemandExportDialog',
        'Capybara.ajax.BulkExportOnDemandExportsService'
    ],

    refs: [{
        ref: 'downloadManagerGrid',
        selector: 'downloadManagerGrid'
    }, {
        ref: 'navigationMenu',
        selector: 'navigationMenu'
    }, {
        ref: 'bulkExportOnDemandDetailDialog',
        selector: 'bulkExportOnDemandDetailDialog'
    }, {
        ref: 'bulkExportOnDemandExportDialog',
        selector: 'bulkExportOnDemandExportDialog'
    }],

    i18n: {
        approvalSuccess: 'Export {0} approved',
        denialSuccess: 'Export {0} denied',
        approvalFailure: 'Error: unable to approve export {0}. Please try again.',
        denialFailure: 'Error: unable to deny export {0}. Please try again.'
    },

    init: function () {
        var me = this;

        me.control({
            'downloadManagerGrid': {
                showDetailDialog: me.showExportDetails,
                beforeactivate: me.reloadDownloadManagerGridStore
            },
            'navigationMenu': {
                downloadManagerScreen: me.reloadDownloadManagerGridStore
            },
            'bulkExportOnDemandDetailDialog': {
                startOnDemandDownload: function (event, files) {
                    // TODO: implement or remove it
                },
                showBulkExportOnDemandExportDialog: me.showBulkExportOnDemandExportDialog
            },
            'bulkExportOnDemandExportDialog': {
                authorizeOnDemandExport: me.authorizeOnDemandExport
            }
        });
    },

    showBulkExportOnDemandExportDialog: function (authorizeAction) {
        var me = this,
            exportDetailData = me.getBulkExportOnDemandDetailDialog().getDetailData();

        if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.APPROVE) {
            Ext.create('Capybara.view.dialogs.BulkExportOnDemandExportDialog').showApproveBulkExportOnDemandExportDialog(exportDetailData);
        } else if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.DENY) {
            Ext.create('Capybara.view.dialogs.BulkExportOnDemandExportDialog').showDenyBulkExportOnDemandExportDialog(exportDetailData);
        }
    },

    showExportDetails: function (element, record) {
        var jobId = record.get('jobId');

        Ext.ModelManager.getModel('Capybara.model.BulkExportOnDemandDetail').load(jobId, {
            scope: this,
            success: function (detailRecord) {
                var dialog = Ext.create('Capybara.view.dialogs.BulkExportOnDemandDetailDialog', {
                    detailData: detailRecord
                });
                dialog.center();
                dialog.show();
            },
            failure: function () {
                Ext.error('Loading BulkExportOnDemandDetail failed');
            }
        });
    },

    reloadDownloadManagerGridStore: function () {
        Ext.getStore('bulkExportOnDemandSummaries').load();
        Ext.getStore('bulkExportOnDemandHistorical').load();
    },

    authorizeOnDemandExport: function (authorizeAction) {
        var me = this,
            bulkExportOnDemandAuthorizeRequest = Ext.JSON.encode({
               jobId: me.getBulkExportOnDemandExportDialog().getExportData().get('id'),
               exportName: me.getBulkExportOnDemandDetailDialog().getDetailData().get('exportName'),
               comment: me.getBulkExportOnDemandExportDialog().getComment()
            }),
            approveOnDemandExport;

        if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.APPROVE) {
            approveOnDemandExport = true;
            BulkExportOnDemandExportsService.authorizeOnDemandExport(bulkExportOnDemandAuthorizeRequest,
                    me.handleOnDemandExportAuthorizationSuccess, me.handleOnDemandExportAuthorizationFailure,
                    me, approveOnDemandExport);
        } else if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.DENY) {
            approveOnDemandExport = false;
            BulkExportOnDemandExportsService.authorizeOnDemandExport(bulkExportOnDemandAuthorizeRequest,
                    me.handleOnDemandExportAuthorizationSuccess, me.handleOnDemandExportAuthorizationFailure,
                    me, approveOnDemandExport);
        }
    },

    handleOnDemandExportAuthorizationSuccess: function(response) {
        var me = this,
            responseJSON,
            notification;

        if (Ext.isString(response.responseText)) {
            responseJSON = Ext.JSON.decode(response.responseText);
            me.getBulkExportOnDemandExportDialog().close();
            me.getBulkExportOnDemandDetailDialog().close();
            notification = Capybara.ux.Notification.inProgress();

            if (responseJSON.exportApproved && !responseJSON.exportDenied) {
                notification.success(me._getOnDemandAuthorizationNotificationConfig('approvalSuccess', responseJSON.exportName));
            } else if (responseJSON.exportDenied && !responseJSON.exportApproved) {
                notification.success(me._getOnDemandAuthorizationNotificationConfig('denialSuccess', responseJSON.exportName));
            }
            me.reloadDownloadManagerGridStore();
        }
    },

    handleOnDemandExportAuthorizationFailure: function(response) {
        var me = this,
            options = response.request.options,
            optionsJSON,
            notification;

        optionsJSON = Ext.JSON.decode(options.jsonData);
        me.getBulkExportOnDemandExportDialog().close();
        me.getBulkExportOnDemandDetailDialog().close();
        notification = Capybara.ux.Notification.inProgress();

        if (options.url === '/export/ondemandexports/approve') {
            notification.error(me._getOnDemandAuthorizationNotificationConfig('approvalFailure', optionsJSON.exportName));
        } else if (options.url === '/export/ondemandexports/deny') {
            notification.error(me._getOnDemandAuthorizationNotificationConfig('denialFailure', optionsJSON.exportName));
        }
    },

    _getOnDemandAuthorizationNotificationConfig: function (type, exportName) {
        return {
            controller: this,
            i18n: type,
            params: [exportName]
        };
    }

});
