Ext.define('Capybara.controller.DownloadManager', {
    extend: 'Ext.app.Controller',

    requires: [
        'Capybara.view.bulkexport.DownloadManagerGrid',
        'Capybara.view.navigation.NavigationMenu',
        'Capybara.ux.Notification',
        'Capybara.util.Utility',
        'Capybara.app.Constants',
        'Capybara.view.dialogs.BulkExportOnDemandDetailDialog',
        'Capybara.view.dialogs.BulkExportOnDemandExportDialog',
        'Capybara.ajax.BulkExportOnDemandService'
    ],

    refs: [{
        ref: 'downloadManagerGrid',
        selector: 'downloadManagerGrid'
    }, {
        ref: 'navigationMenu',
        selector: 'navigationMenu'
    }, {
        ref: 'bulkExportOnDemandDetailDialog',
        selector: 'bulkExportOnDemandDetailDialog'
    }, {
        ref: 'bulkExportOnDemandExportDialog',
        selector: 'bulkExportOnDemandExportDialog'
    }],

    i18n: {
        approvalSuccess: 'Export {0} approved',
        denialSuccess: 'Export {0} denied',
        alreadyApprovedFailure: 'Error: export {0} has already been approved.',
        alreadyDeniedFailure: 'Error: export {0} has already been denied.',
        approvalFailure: 'Error: unable to approve export {0}. Please try again.',
        denialFailure: 'Error: unable to deny export {0}. Please try again.',
        directDownloadApproverRightsErrorTitle: 'Error: Direct Download Failed',
        directDownloadApproverRightsChangedErrorDescription: 'You can not use the Direct download option due to access restrictions on your account.  To use the Direct download option you must contact your administrator.',
        closeBtnText: 'Close'
    },

    statics: {
        ON_DEMAND_BULK_EXTRACTION_ALEADY_APPROVED_DENIED: 'ON_DEMAND_BULK_EXTRACTION_ALEADY_APPROVED_DENIED',
        ON_DEMAND_BULK_EXTRACTION_APPROVER_NO_LONGER_VALID: 'ON_DEMAND_BULK_EXTRACTION_APPROVER_NO_LONGER_VALID',
        SERVER_DOWN: 'Server is down'
    },

    init: function () {
        var me = this;

        me.control({
            'downloadManagerGrid': {
                showDetailDialog: me.showExportDetails,
                beforeactivate: me.reloadDownloadManagerGridStore
            },
            'navigationMenu': {
                downloadManagerScreen: me.reloadDownloadManagerGridStore
            },
            'bulkExportOnDemandDetailDialog': {
                startOnDemandDownload: function (event, files) {
                    // TODO: implement or remove it
                },
                showBulkExportOnDemandExportDialog: me.showBulkExportOnDemandExportDialog
            },
            'bulkExportOnDemandExportDialog': {
                authorizeOnDemandExport: me.authorizeOnDemandExport
            }
        });
    },

    showBulkExportOnDemandExportDialog: function (authorizeAction) {
        var me = this,
            exportDetailData = me.getBulkExportOnDemandDetailDialog().getDetailData();

        if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.APPROVE) {
            Ext.create('Capybara.view.dialogs.BulkExportOnDemandExportDialog').showApproveBulkExportOnDemandExportDialog(exportDetailData);
        } else if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.DENY) {
            Ext.create('Capybara.view.dialogs.BulkExportOnDemandExportDialog').showDenyBulkExportOnDemandExportDialog(exportDetailData);
        }
    },

    showExportDetails: function (element, record) {
        var jobId = record.get('jobId');

        Ext.ModelManager.getModel('Capybara.model.BulkExportOnDemandDetail').load(jobId, {
            scope: this,
            success: function (detailRecord) {
                var dialog = Ext.create('Capybara.view.dialogs.BulkExportOnDemandDetailDialog', {
                    detailData: detailRecord
                });
                dialog.center();
                dialog.show();
            },
            failure: function () {
                Ext.error('Loading BulkExportOnDemandDetail failed');
            }
        });
    },

    reloadDownloadManagerGridStore: function () {
        Ext.getStore('bulkExportOnDemandSummaries').load();
        Ext.getStore('bulkExportOnDemandHistorical').load();
    },

    authorizeOnDemandExport: function (authorizeAction) {
        var me = this,
            bulkExportOnDemandAuthorizeRequest = Ext.JSON.encode({
               jobId: me.getBulkExportOnDemandExportDialog().getExportData().get('id'),
               exportName: me.getBulkExportOnDemandDetailDialog().getDetailData().get('exportName'),
               comment: me.getBulkExportOnDemandExportDialog().getComment()
            }),
            approveOnDemandExport;

        if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.APPROVE) {
            approveOnDemandExport = true;
            BulkExportOnDemandService.authorizeOnDemandExport(bulkExportOnDemandAuthorizeRequest,
                    me.handleOnDemandExportAuthorizationSuccess, me.handleOnDemandExportAuthorizationFailure,
                    me, approveOnDemandExport);
        } else if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.DENY) {
            approveOnDemandExport = false;
            BulkExportOnDemandService.authorizeOnDemandExport(bulkExportOnDemandAuthorizeRequest,
                    me.handleOnDemandExportAuthorizationSuccess, me.handleOnDemandExportAuthorizationFailure,
                    me, approveOnDemandExport);
        }
    },

    handleOnDemandExportAuthorizationSuccess: function(response) {
        var me = this,
            responseJSON,
            notification;

        if (Ext.isString(response.responseText)) {
            responseJSON = Ext.JSON.decode(response.responseText);
            me.getBulkExportOnDemandExportDialog().close();
            me.getBulkExportOnDemandDetailDialog().close();
            notification = Capybara.ux.Notification.inProgress();

            if (responseJSON.exportApproved && !responseJSON.exportDenied) {
                notification.success(me._getOnDemandAuthorizationNotificationConfig('approvalSuccess', responseJSON.exportName));
            } else if (responseJSON.exportDenied && !responseJSON.exportApproved) {
                notification.success(me._getOnDemandAuthorizationNotificationConfig('denialSuccess', responseJSON.exportName));
            }
            me.reloadDownloadManagerGridStore();
        }
    },

    handleOnDemandExportAuthorizationFailure: function(response) {
        var me = this,
            options = response.request.options,
            optionsJSON,
            notification,
            responseText;

        optionsJSON = Ext.JSON.decode(options.jsonData);
        me.getBulkExportOnDemandExportDialog().close();
        me.getBulkExportOnDemandDetailDialog().close();
        responseText = response.responseText;

        if (responseText === me.statics().ON_DEMAND_BULK_EXTRACTION_APPROVER_NO_LONGER_VALID) {
            me._showApproverAccessDeniedDialog();
        } else {
            notification = Capybara.ux.Notification.inProgress();
            if (options.url === BulkExportOnDemandService.APPROVE_URL) {
                if (responseText === me.statics().ON_DEMAND_BULK_EXTRACTION_ALEADY_APPROVED_DENIED) {
                    notification.error(me._getOnDemandAuthorizationNotificationConfig('alreadyApprovedFailure', optionsJSON.exportName));
                } else if (responseText === me.statics().SERVER_DOWN) {
                    notification.error(me._getOnDemandAuthorizationNotificationConfig('approvalFailure', optionsJSON.exportName));
                }
            } else if (options.url === BulkExportOnDemandService.DENY_URL) {
                if (response.responseText === me.statics().ON_DEMAND_BULK_EXTRACTION_ALEADY_APPROVED_DENIED) {
                    notification.error(me._getOnDemandAuthorizationNotificationConfig('alreadyDeniedFailure', optionsJSON.exportName));
                } else if (responseText === me.statics().SERVER_DOWN) {
                    notification.error(me._getOnDemandAuthorizationNotificationConfig('denialFailure', optionsJSON.exportName));
                }
            }
        }
        me.reloadDownloadManagerGridStore();
    },

    _getOnDemandAuthorizationNotificationConfig: function (type, exportName) {
        return {
            controller: this,
            i18n: type,
            params: [exportName]
        };
    },

    _showApproverAccessDeniedDialog: function () {
        var me = this;
        // Display error dialog
        var dialog = Ext.create('Capybara.view.dialogs.ErrorDialog', {
            title: me.i18n.directDownloadApproverRightsErrorTitle,
            actionButtonText: me.i18n.closeBtnText,
            bodyText: me.i18n.directDownloadApproverRightsChangedErrorDescription
        });
        dialog.center();
        dialog.show();
    }

});
