Ext.define('Capybara.controller.DownloadManager', {
    extend: 'Ext.app.Controller',

    requires: [
        'Capybara.view.bulkexport.DownloadManagerGrid',
        'Capybara.view.navigation.NavigationMenu',
        'Capybara.ux.Notification',
        'Capybara.util.Utility',
        'Capybara.app.Constants',
        'Capybara.view.dialogs.BulkExportOnDemandDetailDialog',
        'Capybara.view.dialogs.BulkExportOnDemandExportDialog',
        'Capybara.ajax.BulkExportOnDemandExportsService'
    ],

    refs: [{
        ref: 'downloadManagerGrid',
        selector: 'downloadManagerGrid'
    }, {
        ref: 'navigationMenu',
        selector: 'navigationMenu'
    }, {
        ref: 'bulkExportOnDemandDetailDialog',
        selector: 'bulkExportOnDemandDetailDialog'
    }, {
        ref: 'bulkExportOnDemandExportDialog',
        selector: 'bulkExportOnDemandExportDialog'
    }],

    i18n: {
        approvalSuccess: 'Export {0} approved',
        denialSuccess: 'Export {0} denied',
        approvalFailure: 'Error: unable to approve export {0}. Please try again.',
        denialFailure: 'Error: unable to deny export {0}. Please try again.',
        exitWhileDownloadingWarning: 'Warning! Download may be in progress. Exiting while a download is in progress will abandon the download.'
    },

    init: function () {
        var me = this;

        me.control({
            'downloadManagerGrid': {
                showDetailDialog: me.showExportDetails,
                beforeactivate: me.reloadDownloadManagerGridStore
            },
            'navigationMenu': {
                downloadManagerScreen: me.reloadDownloadManagerGridStore
            },
            'bulkExportOnDemandDetailDialog': {
                startOnDemandDownload: me.startOnDemandDownload,
                showBulkExportOnDemandExportDialog: me.showBulkExportOnDemandExportDialog
            },
            'bulkExportOnDemandExportDialog': {
                authorizeOnDemandExport: me.authorizeOnDemandExport
            }
        });
        Ext.EventManager.on(window, 'beforeunload', me.manageWindowClose, me, {
            normalized:false //we need this for firefox
        });
        //timestamps of downloads that are queued for download, but may not yet have had cookies created on the web server
        //The key to an entry will be the time the download is attempted.  The value of each entry represents the time a download
        //may expire.  A download may expire if a) something takes too long or b) the download failed to at some point before the server could create
        //a download cookie.  Basically, there's a set amount of time by which a cookie must be created, and if it's not, the download monitoring considers
        //the download to be aborted
        me.queuedDownloadIds = {};
        me.delayManageDownloadCookiesTask = new Ext.util.DelayedTask(me._manageDownloadCookies, me);
    },

    showBulkExportOnDemandExportDialog: function (authorizeAction) {
        var me = this,
            exportDetailData = me.getBulkExportOnDemandDetailDialog().getDetailData();

        if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.APPROVE) {
            Ext.create('Capybara.view.dialogs.BulkExportOnDemandExportDialog').showApproveBulkExportOnDemandExportDialog(exportDetailData);
        } else if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.DENY) {
            Ext.create('Capybara.view.dialogs.BulkExportOnDemandExportDialog').showDenyBulkExportOnDemandExportDialog(exportDetailData);
        }
    },

    showExportDetails: function (element, record) {
        var jobId = record.get('jobId');

        Ext.ModelManager.getModel('Capybara.model.BulkExportOnDemandDetail').load(jobId, {
            scope: this,
            success: function (detailRecord) {
                var dialog = Ext.create('Capybara.view.dialogs.BulkExportOnDemandDetailDialog', {
                    detailData: detailRecord
                });
                dialog.center();
                dialog.show();
            },
            failure: function () {
                Ext.error('Loading BulkExportOnDemandDetail failed');
            }
        });
    },

    reloadDownloadManagerGridStore: function () {
        Ext.getStore('bulkExportOnDemandSummaries').load();
        Ext.getStore('bulkExportOnDemandHistorical').load();
    },

    authorizeOnDemandExport: function (authorizeAction) {
        var me = this,
            bulkExportOnDemandAuthorizeRequest = Ext.JSON.encode({
               jobId: me.getBulkExportOnDemandExportDialog().getExportData().get('id'),
               exportName: me.getBulkExportOnDemandDetailDialog().getDetailData().get('exportName'),
               comment: me.getBulkExportOnDemandExportDialog().getComment()
            }),
            approveOnDemandExport;

        if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.APPROVE) {
            approveOnDemandExport = true;
            BulkExportOnDemandExportsService.authorizeOnDemandExport(bulkExportOnDemandAuthorizeRequest,
                    me.handleOnDemandExportAuthorizationSuccess, me.handleOnDemandExportAuthorizationFailure,
                    me, approveOnDemandExport);
        } else if (authorizeAction === Constants.ONDEMAND_EXPORT_AUTHORIZATION_STATES.DENY) {
            approveOnDemandExport = false;
            BulkExportOnDemandExportsService.authorizeOnDemandExport(bulkExportOnDemandAuthorizeRequest,
                    me.handleOnDemandExportAuthorizationSuccess, me.handleOnDemandExportAuthorizationFailure,
                    me, approveOnDemandExport);
        }
    },

    handleOnDemandExportAuthorizationSuccess: function(response) {
        var me = this,
            responseJSON,
            notification;

        if (Ext.isString(response.responseText)) {
            responseJSON = Ext.JSON.decode(response.responseText);
            me.getBulkExportOnDemandExportDialog().close();
            me.getBulkExportOnDemandDetailDialog().close();
            notification = Capybara.ux.Notification.inProgress();

            if (responseJSON.exportApproved && !responseJSON.exportDenied) {
                notification.success(me._getOnDemandAuthorizationNotificationConfig('approvalSuccess', responseJSON.exportName));
            } else if (responseJSON.exportDenied && !responseJSON.exportApproved) {
                notification.success(me._getOnDemandAuthorizationNotificationConfig('denialSuccess', responseJSON.exportName));
            }
            me.reloadDownloadManagerGridStore();
        }
    },

    handleOnDemandExportAuthorizationFailure: function(response) {
        var me = this,
            options = response.request.options,
            optionsJSON,
            notification;

        optionsJSON = Ext.JSON.decode(options.jsonData);
        me.getBulkExportOnDemandExportDialog().close();
        me.getBulkExportOnDemandDetailDialog().close();
        notification = Capybara.ux.Notification.inProgress();

        if (options.url === '/export/ondemandexports/approve') {
            notification.error(me._getOnDemandAuthorizationNotificationConfig('approvalFailure', optionsJSON.exportName));
        } else if (options.url === '/export/ondemandexports/deny') {
            notification.error(me._getOnDemandAuthorizationNotificationConfig('denialFailure', optionsJSON.exportName));
        }
    },

    _getOnDemandAuthorizationNotificationConfig: function (type, exportName) {
        return {
            controller: this,
            i18n: type,
            params: [exportName]
        };
    },

    startOnDemandDownload: function (jobId, messageStoreId) {
        var downloadAttemptTime = new Date().getTime(),
            downloadId = downloadAttemptTime,
            downloadUrl = '/export/download/' + jobId + '/' + messageStoreId + '?downloadId=' + downloadId;

        this.queuedDownloadIds[downloadId] = downloadAttemptTime + 2 * 60 * 1000; //expire 2 minutes from now
        //We want the iframe to be a child of body, so that if the download dialog is closed, the iframe is not destroyed,
        //which will abort the download
        bodyEl = Ext.getBody();
        bodyEl.createChild({tag: 'iframe', src: downloadUrl, id: downloadId});
        this.delayManageDownloadCookiesTask.delay(5000);
    },

    _manageDownloadCookies: function () {
        BulkExportOnDemandExportsService.getInProgressOnDemandDownloadIds(this.handleGetOnDemandExportDownloadIdsSuccess,
                this.handleGetOnDemandExportDownloadIdsFailures, this);
    },

    handleGetOnDemandExportDownloadIdsSuccess: function(response) {
        var me = this,
            activeDownloadIds,
            downloadCookieIds,
            cookiesToDelete = [],
            clonedQueuedDownloadIds,
            downloadId,
            currentTime = new Date().getTime(),
            iframesToRemove = [];

        if (Ext.isString(response.responseText)) {
            activeDownloadIds = Ext.JSON.decode(response.responseText);
            downloadCookieIds = me._getDownloadCookieIds();
            //Remove queued download ids for which there is a download cookie
            Ext.Array.each(downloadCookieIds, function(downloadCookieId) {
                if (this.queuedDownloadIds[downloadCookieId]) {
                    delete this.queuedDownloadIds[downloadCookieId];
                }
            }, me);
            //Remove expired queued download ids & mark iframes for removal
            clonedQueuedDownloadIds = Ext.clone(this.queuedDownloadIds);
            for (downloadId in clonedQueuedDownloadIds) {
                if (currentTime > this.queuedDownloadIds[downloadId]) {
                    delete this.queuedDownloadIds[downloadId];
                    iframesToRemove.push(downloadId);
                }
            }
            //Mark for deletion cookies that are no longer active.  Also mark iframes for removal
            Ext.Array.each(downloadCookieIds, function(downloadCookieId) {
                if (!activeDownloadIds[downloadCookieId]) {
                    cookiesToDelete.push(downloadCookieId);
                    iframesToRemove.push(downloadCookieId);
                }
            }, me);
            //Actually delete the marked cookies
            Ext.Array.each(cookiesToDelete, function(cookieIdToDelete) {
                Ext.util.Cookies.clear('bulkExportDownload-' + cookieIdToDelete, '/', '.' + location.hostname);
            }, me);
            //And remove any iframes marked for deletion
            Ext.Array.each(iframesToRemove, function(iframeToRemove) {
                var iframeEl = Ext.query('iframe#' + iframeToRemove);
                if (!Ext.isEmpty(iframeEl)) {
                    Ext.removeNode(iframeEl[0]);
                }
            }, me);
            //Any active cookies or queued download ids? If so, we need to retry again in a bit
            if (!Utility.isObjectEmpty(activeDownloadIds) || !Utility.isObjectEmpty(this.queuedDownloadIds)) {
                this.delayManageDownloadCookiesTask.delay(15000);
            }
        }
    },

    handleGetOnDemandExportDownloadIdsFailures: function(response) {
        Ext.Error('Failed to get active download ids');
    },

    manageWindowClose: function (e) {
        var downloadCookieIds = this._getDownloadCookieIds(),
            me = this,
            message = me.i18n.exitWhileDownloadingWarning;

        if (downloadCookieIds.length > 0 || !Utility.isObjectEmpty(this.queuedDownloadIds)) {
            //These cookies or queued download ids could be stale, so the dialog text is worded such that it
            //indicates that the user *may* have downloads in progress.  The periodic cookie checking code will clean up
            //any stale cookies or queued download ids, eventually
            if (e) {
                e.returnValue = message;
            }
            if (window.event) {
                window.event.returnValue = message;
            }
            return message;
        }
    },

    //Loosely based of Ext.state.CookieProvider.readCookies
    _getDownloadCookieIds : function(){
        var cookies = [],
            c = document.cookie + ";",
            re = /\s?(.*?)=(.*?);/g,
            prefix = 'bulkExportDownload-',
            len = prefix.length,
            matches,
            name;

        while ((matches = re.exec(c)) !== null) {
            name = matches[1];
            if (name && name.substring(0, len) == prefix) {
                cookies.push(name.substr(len));
            }
        }
        return cookies;
    }
});
